{% set info = h.files_link_details(data[field.field_name]) %}
{% set cls = field.files_object_class %}


{% if info and info.href %}
    <object class="{{ cls }}" data="{{ info.href }}" type="{{ info.content_type }}"></object>

{% endif %}
