{% set info = h.files_link_details(data[field.field_name]) %}
{% set cls = field.files_image_class or "img-thumbnail" %}

{% if info and info.href %}
    <img class="{{ cls }}" alt="{{ info.label }}" src="{{ info.href }}"/>

{% elif field.files_image_placeholder %}
    <img class="{{ cls }}" alt="{{ _('Image not found') }}" src="{{ field.files_image_placeholder }}"/>

{% endif %}
